using System;
using System.Collections;
using System.Collections.Generic;
using System.Dynamic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Text.Json;
using System.Threading.Tasks;
using Azure;
using Azure.Cosmos;
using Azure.Storage.Sas;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Cryptography.KeyDerivation;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using TEAMModelOS.Models;
using TEAMModelOS.SDK.DI;
using TEAMModelOS.SDK.Extension;
using TEAMModelOS.SDK.Models;

namespace TEAMModelOS.Controllers
{
    [Route("student")]
    [ApiController]
    public class StudentController : Controller
    {
        private readonly AzureCosmosFactory _azureCosmos;
        private readonly AzureStorageFactory _azureStorage;
        private readonly DingDing _dingDing;
        private readonly Option _option;

        public StudentController(
            AzureCosmosFactory azureCosmos,
            AzureStorageFactory azureStorage,
            DingDing dingDing,
            IOptionsSnapshot<Option> option
            )
        {
            _azureCosmos = azureCosmos;
            _azureStorage = azureStorage;
            _dingDing = dingDing;
            _option = option?.Value;
        }

        /// <summary>
        /// 學生帳號管理
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [AllowAnonymous]
        [HttpPost("student-manage")]
        public async Task<IActionResult> StudentManage(JsonElement request)
        {
            try
            {
                //TODO : 權限檢查、學校檢查。
                if (!request.TryGetProperty("grant_type", out JsonElement grant_type) || !request.TryGetProperty("schoolId", out JsonElement schoolId)) return BadRequest();

                switch (grant_type.GetString())
                {
                    case "create":
                        //單人創建 創建學生->將學生加入教室 檢查學生ID是否重複，欲加入的班級存不存在，座號是否重複。
                        //id pw name classId no year
                        //retrun 如果有重複則回{ existNo } , 成功則{ id, name, year, classId, no }
                        var importStuds = request.GetProperty("students").EnumerateArray();
                        while (importStuds.MoveNext())
                        {
                            JsonElement currStud = importStuds.Current;
                            string id = null, name = null, pw = null, year = null, no = null, classId = null;

                            //讀取輸入的資料
                            if (!currStud.TryGetProperty("id", out var tmpId) || !currStud.TryGetProperty("name", out var tmpName)) continue;
                            id = tmpId.GetString();
                            name = tmpName.GetString();
                            if (currStud.TryGetProperty("pw", out var tmpPw)) pw = tmpPw.GetString();
                            if (currStud.TryGetProperty("year", out var tmpYear)) year = tmpYear.GetString();
                            if (currStud.TryGetProperty("no", out var tmpNo)) no = tmpNo.GetString();
                            if (currStud.TryGetProperty("classId", out var tmpClassId)) classId = tmpClassId.GetString();

                            //要檢查座號使否已被使用
                            var existNo = await checkStudNo(schoolId.GetString(), classId, new List<string>() { id });
                            if (existNo.Count != 0) return this.Ok(new { code = $"Base-{schoolId.GetString()}", existNo = existNo.Select(o => o.id).ToList() });

                            //建立學生
                            studCreateInfo studCreateInfo = new studCreateInfo(id, name, "M", year, pw, classId, no);
                            var isCreateSuc = await createStudent(schoolId.GetString(), studCreateInfo);
                            if (isCreateSuc) return this.Ok(new { code = $"Base-{schoolId.GetString()}", id, name, year, classId, no });
                            else return this.Ok(new { code = $"Base-{schoolId.GetString()}", errorId = id });
                        }
                        break;
                    case "import":
                        //只有ClassNo可以比對
                        var retUpsert = await upsertStudents(schoolId.GetString(), request.GetProperty("students").EnumerateArray());
                        return this.Ok(new { code = $"Base-{schoolId.GetString()}", students = retUpsert.studs, retUpsert.classDuplNos, retUpsert.errorIds });
                    case "read":
                        //讀取該間學校所有的學生資訊
                        var students = await getAllStudent(schoolId.GetString());
                        return this.Ok(new { code = $"Base-{schoolId.GetString()}", students });
                    case "update":
                        //更新學生資料，批量密碼重置，基本資訊更新(姓名、教室ID、性別、學年及座號)
                        var retUpdate = await updateStudents(schoolId.GetString(), request.GetProperty("students").EnumerateArray());
                        return this.Ok(new { code = $"Base-{schoolId.GetString()}", students = retUpdate.studs, retUpdate.classDuplNos, retUpdate.nonexistentIds, retUpdate.errorNos, retUpdate.errorClassId });
                    case "delete":
                        //刪除學生資料及從教室學生名單內移除該學生
                        var sucDelIds = await deleteStudents(schoolId.GetString(), request.GetProperty("students").EnumerateArray());
                        return this.Ok(new { code = $"Base-{schoolId.GetString()}", ids = sucDelIds });
                    case "remove":
                        //將學生基本資料內的classId、no、groupId及groupName寫入null
                        var retRemove = removeStudentClassInfo(schoolId.GetString(), request.GetProperty("students").EnumerateArray());
                        return this.Ok(new { code = $"Base-{schoolId.GetString()}", ids = retRemove.Result.studs, retRemove.Result.nonexistentIds, retRemove.Result.errorIds });
                    default:
                        return BadRequest();
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/StudentManage()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            return BadRequest();
        }

        /// <summary>
        /// 整理前端匯入的學生資訊
        /// </summary>
        /// <param name="schoolId"></param>
        /// <param name="students"></param>
        /// <returns></returns>
        private (Dictionary<string, (string name, string no, string year, string salt, string pw, string classNo, string className)> studs,
            Dictionary<string, string> classInfo,
            Dictionary<string, List<(string id, string no)>> classStudNo,
            List<string> errorYear,
            List<string> duplId) doSortImpStuds(string schoolId, JsonElement.ArrayEnumerator students)
        {
            //批量匯入 檢查輸入數據 確認座號 確認教室(創建教室) 確認學生存不存在或是要不要更新

            //存放輸入的學生資訊 key:stud id value:學生詳細資料
            Dictionary<string, (string name, string no, string year, string salt, string pw, string classNo, string className)> dicStuds = new Dictionary<string, (string name, string no, string year, string salt, string pw, string classNo, string className)>();

            ////欲加入的教室no
            //List<string> wantJoinClassNo = new List<string>();

            //存放教室資訊用 key:classNo value:className
            Dictionary<string, string> dicClassInfo = new Dictionary<string, string>();
            //存放欲加入該間教室的學生座號清單 key:classNo value:no list
            Dictionary<string, List<(string id, string no)>> dicClassStudNo = new Dictionary<string, List<(string id, string no)>>();
            //存放輸入id重複
            List<string> duplId = new List<string>();
            List<string> errorYear = new List<string>();

            while (students.MoveNext())
            {
                //string id = null, no = null, name = null, year = null, pw = null, classNo = null, className = null;
                JsonElement student = students.Current;

                //ClassNo內的座號
                //欲加入的教室
                //查學生 
                //該間教室的座號與目前欲更新的是否有重複 有些可能是同教室換座號 反正都要將學生讀出來
                //舊學生完整資料+新學生資料

                //進行輸入資料的整理
                if (student.TryGetProperty("id", out var tmpId))
                {
                    string id = tmpId.GetString();
                    //如果id欄位是空的，則跳過該筆資料
                    if (string.IsNullOrWhiteSpace(id)) continue;

                    //輸入的id不應有重複
                    if (dicStuds.ContainsKey(id))
                    {
                        //如果id重複，則將之從整理清單內清除
                        duplId.Add(id);
                        dicStuds.Remove(id);
                    }

                    (string name, string no, string year, string salt, string pw, string classNo, string className) studentInfo = (null, null, null, null, null, null, null);

                    if (student.TryGetProperty("name", out var tmpName) && !string.IsNullOrWhiteSpace(tmpName.GetString())) studentInfo.name = tmpName.GetString();


                    //入學學年為必須，故若是無給值則將之紀錄並跳過該筆資料
                    if (student.TryGetProperty("year", out var tmpYear) && !string.IsNullOrWhiteSpace(tmpYear.GetString())) studentInfo.year = tmpYear.GetString();
                    else
                    {
                        errorYear.Add(id);
                        continue;
                    }

                    //Password，若沒給則使用學號當密碼
                    studentInfo.salt = Utils.CreatSaltString(8);
                    studentInfo.pw = student.TryGetProperty("pw", out var tmpPw) && !string.IsNullOrWhiteSpace(tmpPw.GetString())
                                ? Utils.HashedPassword(tmpPw.GetString(), studentInfo.salt)
                                : Utils.HashedPassword(id, studentInfo.salt);


                    if (student.TryGetProperty("classNo", out var tmpClassNo) && !string.IsNullOrWhiteSpace(tmpClassNo.GetString()))
                    {
                        studentInfo.classNo = tmpClassNo.GetString();

                        //在新建帳號上，應要給classNo才能設定no，但是若只是已存在的帳號要進行座號更新呢? 為避免使用者出錯，故已存在的帳號也應當給classNo。
                        if (student.TryGetProperty("no", out var tmpNo) && !string.IsNullOrWhiteSpace(tmpNo.GetString()))
                        {
                            studentInfo.no = tmpNo.GetString();

                            //這邊先將該間教室欲使用到的no整理出來
                            if (dicClassStudNo.ContainsKey(tmpClassNo.GetString())) dicClassStudNo[tmpClassNo.GetString()].Add((id, tmpNo.GetString()));
                            else dicClassStudNo.Add(tmpClassNo.GetString(), new List<(string id, string no)>() { (id, tmpNo.GetString()) });
                        }

                        //有給classNo才會紀錄className，classNo屬於實體教室門牌號，全校理當只會有一個。 
                        if (student.TryGetProperty("className", out var tmpClassName) && !string.IsNullOrWhiteSpace(tmpClassName.GetString()))
                        {
                            studentInfo.className = tmpClassName.GetString();
                            if (!dicClassInfo.ContainsKey(tmpClassNo.GetString())) dicClassInfo.Add(tmpClassNo.GetString(), tmpClassName.GetString());
                        }
                    }

                    //將最後結果加到字典內
                    dicStuds.Add(id, studentInfo);
                }
            }
            return (dicStuds, dicClassInfo, dicClassStudNo, errorYear, duplId);
        }

        private async Task<(List<object> studs, Dictionary<string ,List<string>> classDuplNos, List<string> errorIds)> upsertStudents(
            string schoolId,
            JsonElement.ArrayEnumerator students)
        {
            try
            {
                var sortedImpData = doSortImpStuds(schoolId, students);

                List<string> classNos = sortedImpData.classInfo.Select(o => o.Key).ToList();
                //抓到教室資訊
                var classInfos = await getClassInfoUseNo(schoolId, classNos);
                //取出已存在教室的classId，後面查座號要用。

                List<Task> tasks = new List<Task>();
                //Key:ClassNo Value:No 匯入時只有ClassNo
                Dictionary<string, List<(string id, string no)>> classStudNos = new Dictionary<string, List<(string id, string no)>>();

                //Key:ClassNo Value:ClassId 存放教室no及id的變數
                Dictionary<string, (string classId, string className, string periodId, string gradeId)> classNoId = new Dictionary<string, (string classId, string className, string periodId, string gradeId)>();
                foreach (var classInfo in classInfos)
                {
                    classNoId.Add(classInfo.Key,
                        (classInfo.Value.GetProperty("id").GetString(), classInfo.Value.GetProperty("name").GetString(), classInfo.Value.GetProperty("periodId").GetString(), classInfo.Value.GetProperty("gradeId").GetString()));
                    tasks.Add(
                        Task.Run(
                            async () =>
                            {
                                //(id,no)
                                var studNo = await checkStudNo(schoolId, classInfo.Value.GetProperty("id").GetString());
                                //todo:檢查這部分有無問題
                                classStudNos.Add(classInfo.Key, studNo);
                            }));
                }

                //這邊整理出不存在的教室，之後創建新教室用(比對classNo)。
                var nonexistentClass = classNos.Except(classInfos.Select(o => o.Key).ToList());
                foreach (var item in nonexistentClass)
                {
                    var (classId, classNo, className) = await createClassInfo(schoolId, null, sortedImpData.classInfo[item], item);
                    classStudNos.Add(classNo, new List<(string id, string no)>());
                    classNoId.Add(classNo, (classId, null, null, null));
                }

                var taskWhenAll = Task.WhenAll(tasks);
                taskWhenAll.Wait();

                //-------------------------------------------------------------------------

                //建立學生或是更新學生，並且要確認座號是否重複
                //每間教室的全部座號 欲更新的教室座號
                //先新建帳號若出現409則進行資料更新
                
                //紀錄有重複做號的id
                Dictionary<string, List<string>> duplNos = new Dictionary<string, List<string>>();
                List<string> errorIds = new List<string>();
                List<object> retStuds = new List<object>();


                CosmosContainer cosmosContainer = _azureCosmos.GetCosmosClient().GetContainer("TEAMModelOS", "Student");

                //並行處理
                foreach (var stud in sortedImpData.studs)
                {
                    //這邊一樣要確認已存在和欲加入還有欲修改的座號。
                    //欲修改的不會有重複
                    string classId = null;
                    bool isContinue = false;
                    if (!string.IsNullOrWhiteSpace(stud.Value.no) && !string.IsNullOrWhiteSpace(stud.Value.classNo))
                    {
                        classId = classNoId[stud.Value.classNo].classId;
                        (string id, string no) existNoInfo = (null, null);
                        //檢查要更新的座號是否已存在於雲端座號(已被其他同學使用)
                        classStudNos[stud.Value.classNo].ForEach(
                            o =>
                            {
                                if (o.no.Equals(stud.Value.no) && !o.id.Equals(stud.Key))
                                {
                                    existNoInfo = (o.id, o.no);
                                    if (duplNos.ContainsKey(stud.Value.classNo)) duplNos[stud.Value.classNo].Add(stud.Key);
                                    else duplNos.Add(stud.Value.classNo, new List<string>() { stud.Key });
                                }
                            });

                        //如果不是空的代表有座號重覆到，此時要再確認這個重複座號的id，是否存在於這次的更新，而且是要更新座號的。
                        if (!string.IsNullOrWhiteSpace(existNoInfo.id))
                        {
                            isContinue = true;
                            //TODO:同ID同NO下的檢查會有問題
                            //輪巡所有匯入的學生資料，並檢查匯入的座號。
                            sortedImpData.classStudNo[stud.Value.classNo].ForEach(
                                o =>
                                {
                                    if (o.id.Equals(existNoInfo.id) && !o.no.Equals(existNoInfo.no))
                                    {
                                        //可以更新該座號
                                        isContinue = false;
                                        duplNos[stud.Value.classNo].Remove(stud.Key);
                                        return;
                                    }
                                });
                        }
                    }
                    if (isContinue) continue;

                    (string id, string name, string pic, string year, string no, string classId, string classNo, string className, string gradeId, string periodId) tmpStudInfo
                       = (stud.Key, stud.Value.name, null, stud.Value.year, stud.Value.no, null, stud.Value.classNo, null, null, null);

                    using var memoryStream = new MemoryStream();
                    using var writer = new Utf8JsonWriter(memoryStream);
                    writer.WriteStartObject();

                    writer.WriteString("id", stud.Key);
                    writer.WriteString("pk", $"Base");
                    writer.WriteString("code", $"Base-{schoolId}");
                    writer.WriteString("schoolId", schoolId);
                    writer.WriteString("year", stud.Value.year);
                    writer.WriteString("salt", stud.Value.salt);
                    writer.WriteString("pw", stud.Value.pw);
                    if (string.IsNullOrWhiteSpace(stud.Value.name)) writer.WriteNull("name");
                    else writer.WriteString("name", stud.Value.name);
                    writer.WriteString("gender", "M");
                    writer.WriteNull("picture");
                    writer.WriteNull("mail");
                    writer.WriteNull("mobile");
                    writer.WriteNull("country");

                    if (string.IsNullOrWhiteSpace(stud.Value.classNo)) writer.WriteNull("classId");
                    else
                    {
                        writer.WriteString("classId", classId);
                        tmpStudInfo.classId = classId;
                        tmpStudInfo.className = classNoId[stud.Value.classNo].className;
                        tmpStudInfo.gradeId = classNoId[stud.Value.classNo].gradeId;
                        tmpStudInfo.periodId = classNoId[stud.Value.classNo].periodId;
                    }
                    if (string.IsNullOrWhiteSpace(stud.Value.no)) writer.WriteNull("no");
                    else writer.WriteString("no", stud.Value.no);
                    writer.WriteNull("groupId");
                    writer.WriteNull("groupName");

                    writer.WriteEndObject();
                    writer.Flush();

                    var response = await cosmosContainer.CreateItemStreamAsync(memoryStream, new PartitionKey($"Base-{schoolId}"));

                    if (response.Status == (int)HttpStatusCode.Created)
                    {
                        //如果是Created則啥都不做，讓他去下面進行資料的彙整。
                    }
                    //查到已存在的id，則進行基本資料更新。
                    else if (response.Status == (int)HttpStatusCode.Conflict)
                    {
                        response = await _azureCosmos
                                   .GetCosmosClient()
                                   .GetContainer("TEAMModelOS", "Student")
                                   .ReadItemStreamAsync(stud.Key, new PartitionKey($"Base-{schoolId}"));
                        if (response.Status == (int)HttpStatusCode.OK)
                        {
                            memoryStream.Position = 0;
                            using Stream streamId = response.ContentStream;
                            var jsonDoc = await JsonDocument.ParseAsync(streamId);

                            //重置
                            writer.Reset();
                            writer.WriteStartObject();
                            bool isUpPwDone = false;
                            foreach (var element in jsonDoc.RootElement.EnumerateObject())
                            {
                                if (!element.Name.StartsWith("_"))
                                {
                                    switch (element.Name)
                                    {
                                        case "name":
                                            if (string.IsNullOrWhiteSpace(stud.Value.name)) goto default;
                                            else writer.WriteString("name", stud.Value.name);
                                            break;
                                        case "year":
                                            if (string.IsNullOrWhiteSpace(stud.Value.year)) goto default;
                                            else writer.WriteString("year", stud.Value.year);
                                            break;
                                        case "no":
                                            if (string.IsNullOrWhiteSpace(stud.Value.no)) goto default;
                                            else writer.WriteString("no", stud.Value.no);
                                            break;
                                        case "classId":
                                            if (string.IsNullOrWhiteSpace(stud.Value.classNo)) goto default;
                                            else writer.WriteString("classId", classId);
                                            break;
                                        case "pw":
                                        case "salt":
                                            if (!isUpPwDone)
                                            {
                                                writer.WriteString("pw", stud.Value.pw);
                                                writer.WriteString("salt", stud.Value.salt);
                                                isUpPwDone = true;
                                            }
                                            break;
                                        case "picture":
                                            tmpStudInfo.pic = element.Value.GetString();
                                            goto default;
                                        default:
                                            element.WriteTo(writer);
                                            break;
                                    }
                                }
                            }
                            writer.WriteEndObject();
                            writer.Flush();

                            //此處用Replace，不使用Upsert
                            response = await cosmosContainer.ReplaceItemStreamAsync(memoryStream, stud.Key, new PartitionKey($"Base-{schoolId}"));
                            if (response.Status != (int)HttpStatusCode.OK)
                            {
                                errorIds.Add(stud.Key);
                                continue;
                            }
                        }
                        else
                        {
                            errorIds.Add(stud.Key);
                            continue;
                        }
                    }
                    else
                    {
                        errorIds.Add(stud.Key);
                        continue;
                    }

                    //整理輸出用資料
                    retStuds.Add(
                        new
                        {
                            tmpStudInfo.id,
                            tmpStudInfo.name,
                            tmpStudInfo.pic,
                            tmpStudInfo.year,
                            tmpStudInfo.no,
                            tmpStudInfo.classId,
                            tmpStudInfo.classNo,
                            tmpStudInfo.className,
                            tmpStudInfo.gradeId,
                            tmpStudInfo.periodId
                        });
                }
                return (retStuds, duplNos.Where(o => o.Value.Count != 0).ToDictionary(o=>o.Key,o=>o.Value), errorIds);
            }
            catch (Exception ex)
            {
                await _dingDing.SendBotMsg(
                            $"IES5，{_option.Location}，StudentController/upsertStudents()\nex:{ex.Message}",
                            GroupNames.醍摩豆服務運維群組);
            }
            return (null, null, null);
        }

        /// <summary>
        /// 單純建立單一學生
        /// </summary>
        /// <param name="schoolId"></param>
        /// <param name="studCreateInfo"></param>
        private async Task<bool> createStudent(string schoolId, studCreateInfo studCreateInfo)
        {
            try
            {
                using var stream = new MemoryStream();
                using var writer = new Utf8JsonWriter(stream);

                writer.WriteStartObject();
                writer.WriteString("pk", $"Base");
                writer.WriteString("code", $"Base-{schoolId}");
                writer.WriteString("id", studCreateInfo.id);
                if (string.IsNullOrWhiteSpace(studCreateInfo.name)) writer.WriteNull("name");
                else writer.WriteString("name", studCreateInfo.name);
                if (string.IsNullOrWhiteSpace(studCreateInfo.gender)) writer.WriteNull("gender");
                else writer.WriteString("gender", studCreateInfo.gender);
                writer.WriteString("schoolId", schoolId);
                writer.WriteString("year", studCreateInfo.year);
                writer.WriteNull("picture");
                writer.WriteNull("mail");
                writer.WriteNull("mobile");
                writer.WriteNull("country");

                //Password，若沒給則使用學號當密碼
                string salt = Utils.CreatSaltString(8);
                string hashPw = string.IsNullOrWhiteSpace(studCreateInfo.pw) 
                                    ? Utils.HashedPassword(studCreateInfo.id, salt) 
                                    : Utils.HashedPassword(studCreateInfo.pw, salt);
                writer.WriteString("salt", salt);
                writer.WriteString("pw", hashPw);

                if (string.IsNullOrWhiteSpace(studCreateInfo.classId)) writer.WriteNull("classId");
                else writer.WriteString("classId", studCreateInfo.classId);
                if (string.IsNullOrWhiteSpace(studCreateInfo.no)) writer.WriteNull("no");
                else writer.WriteString("no", studCreateInfo.no);
                writer.WriteNull("groupId"); 
                writer.WriteNull("groupName");

                writer.WriteEndObject();
                writer.Flush();
                var response = await _azureCosmos
                                .GetCosmosClient()
                                .GetContainer("TEAMModelOS", "Student")
                                .CreateItemStreamAsync(stream, new PartitionKey($"Base-{schoolId}"));
                if (response.Status == (int)HttpStatusCode.Created || response.Status == (int)HttpStatusCode.OK) return true;
                if (response.Status == (int)HttpStatusCode.Conflict) return false;
                else
                {
                    await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/createStudent()\nCosmosDB Create response status = {response.Status}\nID:{studCreateInfo.id}", GroupNames.醍摩豆服務運維群組);
                    return false;
                }
            }
            catch(Exception ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/createStudent()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
                return false;
            }
        }

        /// <summary>
        /// 生成Class資料
        /// </summary>
        /// <param name="schoolId"></param>
        /// <param name="classId"></param>
        /// <param name="className"></param>
        /// <param name="students"></param>
        /// <returns></returns>
        private async Task<(string classId, string classNo, string className)> createClassInfo(string schoolId, string classId, string className, string classNo)
        {
            //組Class JSON
            try
            {
                string cId = classId;
                using var memoryStream = new MemoryStream();
                using var writer = new Utf8JsonWriter(memoryStream);
                writer.WriteStartObject();
                writer.WriteString("pk", "Class");
                writer.WriteString("code", $"Class-{schoolId}");
                //如果classId是空的，則生成一組GUID。
                if (string.IsNullOrWhiteSpace(classId))
                {
                    cId = Guid.NewGuid().ToString();
                    writer.WriteString("id", cId);
                }
                else writer.WriteString("id", classId);
                if (string.IsNullOrWhiteSpace(classNo)) writer.WriteNull("no");
                else writer.WriteString("no", classNo);
                writer.WriteNull("x");
                writer.WriteNull("y");
                if (string.IsNullOrWhiteSpace(className)) writer.WriteNull("name");
                else writer.WriteString("name", className);

                writer.WritePropertyName("teacher");
                writer.WriteStartObject();
                writer.WriteNull("id");
                writer.WriteNull("name");
                writer.WriteEndObject();

                writer.WriteNull("gradeId");
                writer.WriteNull("periodId");
                writer.WriteNull("sn");
                writer.WriteString("style", "smart");
                writer.WriteString("openType", "1");
                writer.WriteString("scope", "school");
                writer.WriteEndObject();
                writer.Flush();

                var ret = await _azureCosmos.GetCosmosClient().GetContainer("TEAMModelOS", "School").CreateItemStreamAsync(memoryStream, new PartitionKey($"Class-{schoolId}"));
                if (ret.Status != (int)HttpStatusCode.Created)
                {
                    await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/createClassInfo()\nStatus:{ret.Status}\nSchoolId:{schoolId},ClassId:{classId}", GroupNames.醍摩豆服務運維群組);
                }
                return (cId, classNo, className);
            }
            catch (CosmosException ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/createClassInfo()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            catch (Exception ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/createClassInfo()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            return (null,null,null);
        }

        /// <summary>
        /// 使用學校代碼查詢該校所有學生，並且在查詢該學生所屬的教室及座號，支援offset和limit操作已及ContinuationToken，若有ContinuationToken，則會優先使用ContinuationToken。
        /// </summary>
        /// <param name="schoolId"></param>
        /// <param name="byNameOrId">透過Name或Id來查，所以不會管學制、學級和教室</param>
        /// <param name="byPeriod"></param>
        /// <param name="byGrade"></param>
        /// <param name="byClassId"></param>
        /// <param name="offset"></param>
        /// <param name="limit"></param>
        /// <param name="token"></param>
        /// <returns></returns>
        private async Task<(List<object> students, string continuationToken)> getStudents(string schoolId, string byNameOrId = null, string byPeriod = null, string byGrade = null, string byClassId = null, int offset = -1, int limit = -1, string token = default)
        {
            try
            {   //TODO : 進階查詢選項調整 
                //以學校學生角度去抓資料
                List<(string id, string name, string pic, string year)> listStudent = new List<(string id, string name, string pic, string year)>();
                string queryText = $"SELECT c.id, c.name, c.picture, c.year FROM c WHERE c.code = 'Base-{schoolId}'";

                //如果有選擇ClassId的話，則先取得該教室內的學生。
                List<string> searchId = new List<string>();
                if (!string.IsNullOrWhiteSpace(byClassId))
                {
                    var classInfos = await getClassInfoUseId(schoolId, new List<string>() { byClassId });
                    foreach (var classInfo in classInfos)
                    {
                        var students = classInfo.Value.GetProperty("students").EnumerateArray();
                        while (students.MoveNext())
                        {
                            JsonElement stud = students.Current;
                            string id = stud.GetProperty("id").GetString();
                            searchId.Add(id);
                        }
                    }
                    //將使用者過濾classId所取得的學生ID加入sql字串內
                    if (searchId.Count != 0)
                    {
                        queryText = $"{queryText} AND c.id IN ({string.Join(",", searchId.Select(o => $"'{o}'"))})";
                    }
                }

                //if (!string.IsNullOrWhiteSpace(byYear))
                //{
                //    queryText = $"{queryText} AND c.year = '{byYear}'";
                //}

                //檢查是否有接續token及是否要在sql語法內多增加offset及limit
                if (string.IsNullOrWhiteSpace(token))
                {
                    token = default;
                    if (offset != -1 && limit != -1) queryText = $"{queryText} OFFSET {offset} LIMIT {limit}";
                }

                //回傳用ContinuationToken
                string continuationToken = string.Empty;
                //進行學生資料的查詢
                await foreach (var item in _azureCosmos.GetCosmosClient().GetContainer("TEAMModelOS", "Student")
                   .GetItemQueryStreamIterator(
                       queryText: queryText,
                       continuationToken: token,
                       requestOptions: new QueryRequestOptions()
                       { PartitionKey = new PartitionKey($"Base-{schoolId}") }))
                {
                    continuationToken = item.GetContinuationToken();
                    using var json = await JsonDocument.ParseAsync(item.ContentStream);
                    if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                    {
                        var accounts = json.RootElement.GetProperty("Documents").EnumerateArray();
                        while (accounts.MoveNext())
                        {
                            JsonElement account = accounts.Current;
                            listStudent.Add((account.GetProperty("id").GetString(), account.GetProperty("name").GetString(), account.GetProperty("picture").GetString(), account.GetProperty("year").GetString()));
                        }
                    }
                    //單筆查詢上限為100條，所以查完一次即返回，並且給接續token。
                    break;
                }

                //查學生所屬的教室及座號
                List<object> ret = new List<object>();
                //查教室資訊，使用上面的學生id並透過子查詢查詢。
                queryText = $"SELECT c.id, c.name, c.gradeId, c.students FROM c JOIN (SELECT VALUE t FROM t IN c.students WHERE t.id IN ({string.Join(",", listStudent.Select(o => $"'{o.id}'"))}))";
                await foreach (Response item in _azureCosmos.GetCosmosClient().GetContainer("TEAMModelOS", "School")
                   .GetItemQueryStreamIterator(
                   queryText: queryText,
                   //continuationToken: token,
                   requestOptions: new QueryRequestOptions()
                   { PartitionKey = new PartitionKey($"Class-{schoolId}") }))
                {
                    using var json = await JsonDocument.ParseAsync(item.ContentStream);
                    if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                    {
                        var classrooms = json.RootElement.GetProperty("Documents").EnumerateArray();
                        while (classrooms.MoveNext())
                        {
                            JsonElement classroom = classrooms.Current;

                            var studs = classroom.GetProperty("students").EnumerateArray();
                            while (studs.MoveNext())
                            {
                                JsonElement stud = studs.Current;
                                string id = stud.GetProperty("id").GetString();
                                //整理出前端所需的資訊
                                var tmp = listStudent
                                    .Where(o => o.id.Equals(id, StringComparison.Ordinal))
                                    .Select(o =>
                                        new
                                        {
                                            o.id,
                                            o.name,
                                            o.pic,
                                            o.year,
                                            no = stud.GetProperty("no").GetString(),
                                            gradeId = classroom.GetProperty("gradeId").GetString(),
                                            className = classroom.GetProperty("name").GetString()
                                        });
                                ret.AddRange(tmp);
                                //刪除已整理完的ID
                                listStudent.RemoveAll(o => o.id.Equals(id, StringComparison.Ordinal));
                            }
                        }
                    }
                }
                var notJoinClassStuds = listStudent.Select(o =>
                                        new
                                        {
                                            o.id,
                                            o.name,
                                            o.pic,
                                            o.year,
                                            no = (string)null,
                                            gradeId = (string)null,
                                            className = (string)null
                                        });
                ret.AddRange(notJoinClassStuds);
                return (ret, continuationToken);
            }
            catch (CosmosException ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/getStudents()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            catch (Exception ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/getStudents()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            return (null, null);
        }

        /// <summary>
        /// 取得該學校的所有學生。
        /// </summary>
        /// <param name="schoolId"></param>
        /// <returns> [{id,name,pic,year,no,classId,classNo,className,gradeId,periodId},{id,name,pic,..}..]</returns>
        private async Task<List<object>> getAllStudent(string schoolId)
        {
            try
            {   
                //TODO : 進階查詢選項調整、部分地方可用並行處理
                //以學校學生角度去抓資料
                Dictionary<string, List<(string id, string name, string pic, string year, string no)>> dicClassStuds = new Dictionary<string, List<(string id, string name, string pic, string year, string no)>>();
                List<(string id, string name, string pic, string year, string no)> notJoinClassStuds = new List<(string id, string name, string pic, string year, string no)>();
                
                string queryText = $"SELECT c.id, c.name, c.picture, c.year, c.classId, c.no FROM c WHERE c.code = 'Base-{schoolId}'";

                //回傳用ContinuationToken
                string continuationToken = string.Empty;
                var container = _azureCosmos.GetCosmosClient().GetContainer("TEAMModelOS", "Student");

                //進行學生資料的查詢 TEAMModelOS-Student
                await foreach (var item in _azureCosmos.GetCosmosClient().GetContainer("TEAMModelOS", "Student")
                   .GetItemQueryStreamIterator(
                       queryText: queryText,
                       requestOptions: new QueryRequestOptions()
                       { PartitionKey = new PartitionKey($"Base-{schoolId}"), MaxItemCount = -1 }))
                {
                    continuationToken = item.GetContinuationToken();
                    using var json = await JsonDocument.ParseAsync(item.ContentStream);
                    if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                    {
                        List<(string id, string name, string pic, string year,string no)> students = new List<(string id, string name, string pic, string year, string no)>();

                        var accounts = json.RootElement.GetProperty("Documents").EnumerateArray();
                        while (accounts.MoveNext())
                        {
                            JsonElement acc = accounts.Current;

                            string classId = acc.GetProperty("classId").GetString();

                            if (string.IsNullOrWhiteSpace(classId))
                            {
                                notJoinClassStuds.Add(
                                        (
                                            acc.GetProperty("id").GetString(),
                                            acc.GetProperty("name").GetString(),
                                            acc.GetProperty("picture").GetString(),
                                            acc.GetProperty("year").GetString(),
                                            acc.GetProperty("no").GetString()
                                        )
                                    );
                            }
                            else if(dicClassStuds.ContainsKey(classId))
                            {
                                dicClassStuds[classId].Add(
                                        (
                                            acc.GetProperty("id").GetString(), 
                                            acc.GetProperty("name").GetString(),
                                            acc.GetProperty("picture").GetString(),
                                            acc.GetProperty("year").GetString(),
                                            acc.GetProperty("no").GetString()
                                        )
                                    );
                            }
                            else
                            {
                                dicClassStuds.Add(classId, 
                                        new List<(string id, string name, string pic, string year, string no)>() 
                                        {
                                            (
                                                acc.GetProperty("id").GetString(),
                                                acc.GetProperty("name").GetString(),
                                                acc.GetProperty("picture").GetString(),
                                                acc.GetProperty("year").GetString(),
                                                acc.GetProperty("no").GetString()
                                            )
                                        }
                                    );
                            }
                        }
                    }
                }

                //查學生所屬的教室及座號
                List<object> ret = new List<object>();

                //查教室的資訊，用以取得gradeId,periodId資訊。
                var classInfos = await getClassInfoUseId(schoolId, dicClassStuds.Keys.ToList());
                
                //輪循所有教室學生的資料
                foreach (var classStud in dicClassStuds)
                {
                    string classId = null, classNo = null, className = null, gradeId = null, periodId = null;
                   
                    if (classInfos.ContainsKey(classStud.Key))
                    {
                        classId = classInfos[classStud.Key].GetProperty("id").GetString();
                        classNo = classInfos[classStud.Key].GetProperty("no").GetString();
                        className = classInfos[classStud.Key].GetProperty("name").GetString();
                        gradeId = classInfos[classStud.Key].GetProperty("gradeId").GetString();
                        periodId = classInfos[classStud.Key].GetProperty("periodId").GetString();
                    }

                    var tmp = classStud.Value.Select(o =>
                                     new
                                     {
                                         o.id,
                                         o.name,
                                         o.pic,
                                         o.year,
                                         o.no,
                                         classId,
                                         classNo,
                                         className,
                                         gradeId,
                                         periodId
                                     });
                    ret.AddRange(tmp);
                }

                //彙整沒有加入教室的學生
                notJoinClassStuds.ForEach(o=> ret.Add(
                    new
                    {
                        o.id,
                        o.name,
                        o.pic,
                        o.year,
                        o.no,
                        classId = (string)null,
                        classNo = (string)null,
                        className = (string)null,
                        gradeId = (string)null,
                        periodId = (string)null
                    }));

                return ret;
            }
            catch (CosmosException ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/getStudents()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            catch (Exception ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/getStudents()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            return null;
        }

        /// <summary>
        /// 取得該校所有教室內的名單
        /// </summary>
        /// <param name="schoolId"></param>
        /// <returns></returns>
        public async Task<Dictionary<string, JsonElement>> getClassStudent(string schoolId, string classId = null)
        {
            try
            {
                string queryText = $"SELECT VALUE FROM c WHERE c.";
                //if (!string.IsNullOrWhiteSpace(classId)) queryText += $" AND c.id = '{classId}'";
                Dictionary<string, JsonElement> listStudent = new Dictionary<string, JsonElement>();

                await foreach (Response item in _azureCosmos.GetCosmosClient().GetContainer("TEAMModelOS", "School")
                    .GetItemQueryStreamIterator(queryText: queryText, requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"Class-{schoolId}") }))
                {
                    using var json = await JsonDocument.ParseAsync(item.ContentStream);
                    if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                    {
                        JsonElement.ArrayEnumerator accounts = json.RootElement.GetProperty("Documents").EnumerateArray();
                        while (accounts.MoveNext())
                        {
                            JsonElement account = accounts.Current;
                            string cId = account.GetProperty("id").GetString();
                            var students = account.GetProperty("students").Clone();
                            listStudent.Add(cId, students);
                        }
                    }
                }
                return listStudent;
            }
            catch (Exception ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/getClassStudentAsync()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            return null;
        }

        /// <summary>
        /// 刪除學生，非透過批量刪除方法。
        /// </summary>
        /// <param name="schoolId"></param>
        /// <param name="students"></param>
        /// <returns></returns>
        private async Task<List<string>> deleteStudents(string schoolId, JsonElement.ArrayEnumerator students)
        {
            List<string> sucIds = new List<string>();
            try
            {
                var exceptions = new List<Exception>();

                var container = _azureCosmos.GetCosmosClient().GetContainer("TEAMModelOS", "Student");
                while (students.MoveNext())
                {
                    string id = string.Empty;
                    try
                    {
                        JsonElement student = students.Current;
                        id = student.GetProperty("id").GetString();
                        var ret = await container.DeleteItemStreamAsync(id, new PartitionKey($"Base-{schoolId}"));
                        if (ret.Status == (int)HttpStatusCode.NoContent) sucIds.Add(id);
                    }
                    catch (CosmosException ex)
                    {
                        exceptions.Add(ex);
                    }
                    catch (Exception ex)
                    {
                        exceptions.Add(ex);
                    }
                }
                if (exceptions.Count == 0) return sucIds;
                else if (exceptions.Count > 1) throw new AggregateException(exceptions);
                else if (exceptions.Count == 1) throw exceptions.Single();
            }
            catch (CosmosException ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/deleteStudents()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            catch (Exception ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/deleteStudents()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            return sucIds;
        }

        /// <summary>
        /// 將學生基本資料內的classId、no、groupId及groupName清為null。
        /// </summary>
        /// <param name="schoolId"></param>
        /// <param name="students">["id1","id2",...]</param>
        /// <returns></returns>
        private async Task<(List<string> studs, List<string> nonexistentIds, List<string> errorIds)> removeStudentClassInfo(string schoolId, JsonElement.ArrayEnumerator students)
        {
            //紀錄輸入的學生
            List<string> impStuds = new List<string>();
            //紀錄更新成功的學生
            List<string> sucStuds = new List<string>();
            //記錄沒查到的學生
            List<string> nonexistentIds = new List<string>();
            //紀錄更新出錯的學生
            List<string> errorIds = new List<string>();

            //整理輸入的學生資訊
            while (students.MoveNext())
            {
                JsonElement student = students.Current;
                impStuds.Add(student.GetString());
            }

            if (impStuds.Count == 0) return (null, null, null);

            string queryText = $"SELECT VALUE c FROM c WHERE c.id IN ({string.Join(",", impStuds.Select(o => $"'{o}'"))})";
            await foreach (Response item in _azureCosmos.GetCosmosClient().GetContainer("TEAMModelOS", "Student")
                            .GetItemQueryStreamIterator(
                                queryText: queryText,
                                requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"Base-{schoolId}") }))
            {
                using var json = await JsonDocument.ParseAsync(item.ContentStream);
                if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                {
                    JsonElement.ArrayEnumerator docs = json.RootElement.GetProperty("Documents").EnumerateArray();
                    while (docs.MoveNext())
                    {
                        JsonElement doc = docs.Current;

                        doc.TryGetProperty("id", out var tmpId);
                        var id = tmpId.GetString();

                        using var stream = new MemoryStream();
                        using var writer = new Utf8JsonWriter(stream);

                        writer.WriteStartObject();
                        foreach (var element in doc.EnumerateObject())
                        {
                            //將教室相關欄位清空
                            switch (true)
                            {
                                case bool _ when element.Name.Equals("classId", StringComparison.Ordinal):
                                    writer.WriteNull("classId");
                                    break;
                                case bool _ when element.Name.Equals("no", StringComparison.Ordinal):
                                    writer.WriteNull("no");
                                    break;
                                case bool _ when element.Name.Equals("groupId", StringComparison.Ordinal):
                                    writer.WriteNull("groupId");
                                    break;
                                case bool _ when element.Name.Equals("groupName", StringComparison.Ordinal):
                                    writer.WriteNull("groupName");
                                    break;
                                case bool _ when element.Name.StartsWith("_", StringComparison.Ordinal):
                                    break;
                                default:
                                    element.WriteTo(writer);
                                    break;
                            }
                        }
                        writer.WriteEndObject();
                        writer.Flush();

                        var ret = await _azureCosmos
                                  .GetCosmosClient()
                                  .GetContainer("TEAMModelOS", "Student")
                                  .ReplaceItemStreamAsync(stream, id, new PartitionKey($"Base-{schoolId}"));

                        if (ret.Status == (int)HttpStatusCode.OK)
                        {
                            sucStuds.Add(id);
                        }
                        else
                        {
                            impStuds.Remove(id);
                            errorIds.Add(id);
                            await _dingDing.SendBotMsg(
                                $"IES5，{_option.Location}，StudentController/removeStudentClassInfo()，CosmosDB response:{ret.Status}\nBase-{schoolId}，id:{id}",
                                GroupNames.醍摩豆服務運維群組);
                        }
                    }
                }
            }

            //將impStuds內的資料移除sucStuds及errorIds，所得的結果就是不存在於資料庫的id。
            sucStuds.ForEach(o => impStuds.Remove(o));
            errorIds.ForEach(o => impStuds.Remove(o));

            return (sucStuds, impStuds, errorIds);
        }

        /// <summary>
        /// 將學生從教室內移除
        /// </summary>
        /// <param name="schoolId"></param>
        /// <param name="students"></param>
        private async Task removeStudentFromClass(string schoolId, JsonElement.ArrayEnumerator students)
        {
            try
            {
                Dictionary<string, List<string>> classStudent = new Dictionary<string, List<string>>();
                List<string> studs = new List<string>();
                //整理教室學生資訊
                while (students.MoveNext())
                {
                    JsonElement student = students.Current;
                    string id, classId = string.Empty;
                    if (student.TryGetProperty("id", out var tmpId) && !string.IsNullOrWhiteSpace(tmpId.GetString()))
                    {
                        id = tmpId.GetString();
                        studs.Add(id);
                    }
                    else continue;

                    if (student.TryGetProperty("classId", out var tmpClassId) && !string.IsNullOrWhiteSpace(tmpClassId.GetString()))
                    {
                        classId = tmpClassId.GetString();
                        if (!string.IsNullOrWhiteSpace(classId) && classStudent.ContainsKey(classId)) classStudent[classId].Add(id);
                        else classStudent.Add(classId, new List<string>() { id });
                    }
                }

                if (studs.Count != 0)
                {
                    //使用子查詢來查詢students欄位裡面是否有相符的學生
                    var queryText = $"SELECT VALUE c FROM c JOIN (SELECT VALUE t FROM t IN c.students WHERE t.id IN ({string.Join(",", studs.Select(o => $"'{o}'"))}))";
                    await foreach (Response item in _azureCosmos.GetCosmosClient().GetContainer("TEAMModelOS", "School")
                                    .GetItemQueryStreamIterator(
                                        queryText: queryText,
                                        requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"Class-{schoolId}") }))
                    {
                        using var json = await JsonDocument.ParseAsync(item.ContentStream);
                        if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                        {
                            JsonElement.ArrayEnumerator docs = json.RootElement.GetProperty("Documents").EnumerateArray();
                            while (docs.MoveNext())
                            {
                                JsonElement doc = docs.Current;

                                doc.TryGetProperty("id", out var tmpId);
                                var id = tmpId.GetString();

                                using var memoryStream = new MemoryStream();
                                using var writer = new Utf8JsonWriter(memoryStream);
                                writer.WriteStartObject();
                                foreach (var element in doc.EnumerateObject())
                                {
                                    if (element.Name.Equals("students", StringComparison.Ordinal))
                                    {
                                        writer.WritePropertyName("students");
                                        writer.WriteStartArray();
                                        var s = element.Value.EnumerateArray();
                                        while (s.MoveNext())
                                        {
                                            JsonElement stud = s.Current;
                                            string sutdId = stud.GetProperty("id").GetString();
                                            if (!studs.Contains(sutdId))
                                            {
                                                string name = stud.GetProperty("name").GetString();
                                                string no = stud.GetProperty("no").GetString();

                                                writer.WriteStartObject();
                                                writer.WriteString("id", sutdId);
                                                if (string.IsNullOrWhiteSpace(name)) writer.WriteNull("name");
                                                else writer.WriteString("name", name);
                                                if (string.IsNullOrWhiteSpace(no)) writer.WriteNull("no");
                                                else writer.WriteString("no", no);
                                                writer.WriteEndObject();
                                            }
                                        }
                                        writer.WriteEndArray();
                                    }
                                    else
                                    {
                                        element.WriteTo(writer);
                                    }
                                }
                                writer.WriteEndObject();
                                writer.Flush();

                                var ret = await _azureCosmos
                                    .GetCosmosClient()
                                    .GetContainer("TEAMModelOS", "School")
                                    .ReplaceItemStreamAsync(memoryStream, id, new PartitionKey($"Class-{schoolId}"));
                                if (ret.Status != (int)HttpStatusCode.OK)
                                {
                                    await _dingDing.SendBotMsg(
                                        $"IES5，{_option.Location}，StudentController/removeStudentFromClass()\nClass-{schoolId}，{string.Join(",", studs.Select(o => $"'{o}'"))}",
                                        GroupNames.醍摩豆服務運維群組);
                                }
                            }
                        }
                    }
                }
            }
            catch (CosmosException ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/removeStudentFromClass()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            catch (Exception ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/removeStudentFromClass()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
        }

        //TODO:檢查學號是否重複，及處理存到CosmosDB錯誤的處理
        /// <summary>
        /// 將學生加入教室，並且會比對id及座號是否有重複，查找教室時該方法是透過classNo進行查詢而非classId。
        /// </summary>
        /// <returns></returns>
        private async Task<
            (Dictionary<string, (string classNo, string periodId, string gradeId, List<string> id)> ret,
            Dictionary<string, List<string>> dicExistId,
            Dictionary<string, List<string>> dicExistNo,
            Dictionary<string, List<string>> dicDuplNo)>
                JoinOrCreateClass(string schoolId, Dictionary<string, List<(string id, string name, string no)>> classStudents, Dictionary<string, string> classInfos)
        {
            try
            {
                //key:教室no value:該間教室錯誤的資訊
                Dictionary<string, List<string>> dicExistId = new Dictionary<string, List<string>>();
                Dictionary<string, List<string>> dicExistNo = new Dictionary<string, List<string>>();
                Dictionary<string, List<string>> dicDuplNo = new Dictionary<string, List<string>>();

                //key:classNo value:學生資訊
                Dictionary<string, (string classId, string periodId, string gradeId, List<string> id)> retClassStud = new Dictionary<string, (string classNo, string periodId, string gradeId, List<string> id)>();

                foreach (var importItem in classStudents)
                {
                    List<string> duplIds = new List<string>();
                    List<string> duplNos = new List<string>();

                    //整理輸入的學生資料
                    List<(string id, string name, string no, string groupId, string groupName)> impStuds
                        = new List<(string id, string name, string no, string groupId, string groupName)>();
                    impStuds.AddRange(importItem.Value.Select(o => (o.id, o.name, o.no, (string)null, (string)null)).ToList());

                    //檢查輸入資料的id及no是否有重複，並且將該資料加入字典內以利後面輸出給前端用。
                    var duplicateId = impStuds.GroupBy(o => o.id).Where(o => o.Count() > 1).Select(o => o.Key).ToList();
                    foreach (var id in duplicateId)
                    {
                        impStuds.RemoveAll(o => o.id.Equals(id));
                        duplIds.Add(id);
                    }
                    dicExistId.Add(importItem.Key, duplIds);

                    var duplicateNo = impStuds.GroupBy(o => o.no).Where(o => o.Count() > 1).Select(o => o.Key).ToList();
                    foreach (var no in duplicateNo)
                    {
                        impStuds.RemoveAll(o => o.no.Equals(no));
                        duplNos.Add(no);
                    }
                    dicExistNo.Add(importItem.Key, duplNos);

                    var tmpImpStuds = new List<(string id, string name, string no, string groupId, string groupName)>();
                    tmpImpStuds.AddRange(impStuds);

                    var queryText = $"SELECT VALUE c FROM c WHERE c.no = '{importItem.Key}'";
                    await foreach (Response response in _azureCosmos.GetCosmosClient().GetContainer("TEAMModelOS", "School")
                                    .GetItemQueryStreamIterator(
                                        queryText: queryText,
                                        requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"Class-{schoolId}") }))
                    {
                        string classId = string.Empty, classNo = string.Empty, periodId = string.Empty, gradeId = string.Empty;

                        //組出新的JSON
                        using var memoryStream = new MemoryStream();
                        using var writer = new Utf8JsonWriter(memoryStream);
                        writer.WriteStartObject();

                        //獲取教室資料，透過classNo。
                        using var json = await JsonDocument.ParseAsync(response.ContentStream);
                        if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                        {
                            JsonElement.ArrayEnumerator docs = json.RootElement.GetProperty("Documents").EnumerateArray();
                            while (docs.MoveNext())
                            {
                                JsonElement existClassData = docs.Current;

                                existClassData.TryGetProperty("id", out var tmpId);
                                classId = tmpId.GetString();
                                existClassData.TryGetProperty("no", out var tmpNo);
                                classNo = tmpNo.GetString();
                                existClassData.TryGetProperty("periodId", out var tmpPeriodId);
                                periodId = tmpPeriodId.GetString();
                                existClassData.TryGetProperty("gradeId", out var tmpGradeId);
                                gradeId = tmpGradeId.GetString();

                                //取出目前該教室的學生清單，若存在則檢查匯入的學生是否已存在，反之則創建新的教室資訊
                                existClassData.TryGetProperty("students", out var existStudents);

                                if (existStudents.GetArrayLength() != 0)
                                {
                                    //若學生數量不為0，代表裡面已有資料，故要檢查是否有重複
                                    List<(string id, string name, string no, string groupId, string groupName)> tmpStuds
                                        = new List<(string id, string name, string no, string groupId, string groupName)>();
                                    //取得雲端教室的名單，並檢查id及座號是否重複
                                    var studs = existStudents.EnumerateArray();

                                    //將雲端的學生名單記錄到tmp內
                                    while (studs.MoveNext())
                                    {
                                        JsonElement stud = studs.Current;
                                        string id = stud.GetProperty("id").GetString();
                                        string no = stud.GetProperty("no").GetString();
                                        string name = stud.GetProperty("name").GetString();
                                        string groupId = stud.GetProperty("groupId").GetString();
                                        string groupName = stud.GetProperty("groupName").GetString();

                                        //可能還是要把欲修改ID加進去，之後再刪除
                                        var existStudInfo = impStuds.Find(o => o.id.Equals(id));
                                        if (existStudInfo.id != null)
                                        {
                                            impStuds.Remove(existStudInfo);
                                            impStuds.Add((existStudInfo.id, existStudInfo.name, existStudInfo.no, groupId, groupName));
                                        }
                                        //else
                                        tmpStuds.Add((id, name, no, groupId, groupName));
                                    }

                                    //雲端與新資料進行比對，並找出沒有要被修改的雲端資料，由於該資料是沒有要修改的，所以其座號是無法變更的，故新資料並不能用到其座號。
                                    var existOldId = tmpStuds.Where(
                                                         a => impStuds.Any(p => !a.id.Equals(p.id))).Select(o => o).ToList();

                                    //將沒有要修改的舊雲端資料與新資料進行比對，確認新資料是否有使用到舊雲端資料已使用的座號。
                                    var errorNo = impStuds.Where(
                                                        a => existOldId.Any(p => a.no.Equals(p.no))).Select(o => o).ToList();
                                    dicDuplNo.Add(classNo, errorNo.Select(o => o.id).ToList());

                                    //去除重複座號資料
                                    foreach (var item in errorNo)
                                    {
                                        impStuds.RemoveAll(p => p.id.Equals(item.id));
                                    }

                                    //去除重複id但no不一樣的舊資料
                                    var doDeleteId = tmpStuds.Where(
                                                        a => impStuds.Any(p => a.id.Equals(p.id))).Select(o => o).ToList();

                                    foreach (var item in doDeleteId)
                                    {
                                        tmpStuds.RemoveAll(p => p.id.Equals(item.id));
                                    }

                                    //將現有資料加回去
                                    impStuds.AddRange(tmpStuds);
                                }

                                foreach (var testDataElement in existClassData.EnumerateObject())
                                {
                                    if (!testDataElement.Name.Equals("students", StringComparison.Ordinal)) testDataElement.WriteTo(writer);
                                }
                            }
                        }
                        else //如果沒有查到資料代表該Class不存在，必須創建。
                        {
                            classId = Guid.NewGuid().ToString();
                            classNo = importItem.Key;
                            writer.WriteString("id", classId);
                            writer.WriteString("code", $"Class-{schoolId}");
                            writer.WriteString("pk", "Class");
                            writer.WriteString("no", importItem.Key);
                            if (string.IsNullOrWhiteSpace(classInfos[importItem.Key])) writer.WriteNull("name");
                            else writer.WriteString("name", classInfos[importItem.Key]);
                            writer.WriteNull("x");
                            writer.WriteNull("y");
                            writer.WritePropertyName("teacher");
                            writer.WriteStartObject();
                            writer.WriteNull("id");
                            writer.WriteNull("name");
                            writer.WriteEndObject();
                            writer.WriteNull("gradeId");
                            writer.WriteNull("periodId");
                            writer.WriteNull("sn");
                            writer.WriteNull("style");
                            writer.WriteNull("timetable");
                            writer.WriteNull("status");
                            writer.WriteString("openType", "1");
                            writer.WriteString("scope", "school");
                            writer.WritePropertyName("courses");
                            writer.WriteStartArray();
                            writer.WriteEndArray();
                        }
                        writer.WritePropertyName("students");
                        writer.WriteStartArray();

                        //進行no排序
                        impStuds = impStuds.OrderBy(o => string.IsNullOrWhiteSpace(o.no)).ThenBy(o => string.IsNullOrEmpty(o.no) ? 0 : int.Parse(o.no)).ToList();
                        foreach (var item in impStuds)
                        {
                            writer.WriteStartObject();
                            writer.WriteString("id", item.id);
                            if (string.IsNullOrWhiteSpace(item.name)) writer.WriteNull("name");
                            else writer.WriteString("name", item.name);
                            if (string.IsNullOrWhiteSpace(item.no)) writer.WriteNull("no");
                            else writer.WriteString("no", item.no);
                            if (string.IsNullOrWhiteSpace(item.groupId)) writer.WriteNull("groupId");
                            else writer.WriteString("groupId", item.groupId);
                            if (string.IsNullOrWhiteSpace(item.groupName)) writer.WriteNull("groupName");
                            else writer.WriteString("groupName", item.groupName);
                            writer.WriteEndObject();
                        }

                        writer.WriteEndArray();

                        writer.WriteEndObject();
                        writer.Flush();


                        //取得學生已加入的舊教室資料。
                        var asd = await getClassInfoUseStudent(schoolId, tmpImpStuds.Select(o => o.id).ToList());


                        var ret = await _azureCosmos
                                        .GetCosmosClient()
                                        .GetContainer("TEAMModelOS", "School")
                                        .UpsertItemStreamAsync(memoryStream, new PartitionKey($"Class-{schoolId}"));

                        retClassStud.Add(classNo, (classId, periodId, gradeId, impStuds.Select(o => o.id).ToList()));
                        
                    }
                }
                return (retClassStud, dicExistId, dicExistNo, dicDuplNo);
            }
            catch (CosmosException ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/joinClass()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            catch (Exception ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/joinClass()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            return (null, null, null, null);
        }

        private async Task<(bool isSuc,List<string> existId, List<string> existNo)> joinClass(string schoolId, string classId, (string id, string name, string no) student)
        {
            var classInfo = await getClassInfoUseId(schoolId, new List<string>() { classId });

            List<(string id, string name, string no, string groupId, string groupName)> existStuds = new List<(string id, string name, string no, string groupId, string groupName)>();
            List<string> existId = new List<string>();
            List<string> existNo = new List<string>();
            classInfo[classId].TryGetProperty("students", out var existStudents);
            var studs = existStudents.EnumerateArray();
            while (studs.MoveNext())
            {
                JsonElement stud = studs.Current;
                string id = stud.GetProperty("id").GetString();
                string name = stud.GetProperty("name").GetString();
                string no = stud.GetProperty("no").GetString();
                string groupId = stud.GetProperty("groupId").GetString();
                string groupName = stud.GetProperty("groupName").GetString();
                existStuds.Add((id, name, no, groupId, groupName));
                if (id.Equals(student.id)) existId.Add(id);
                if (!string.IsNullOrWhiteSpace(no) && no.Equals(student.no)) existNo.Add(no);
            }
            if (existId.Count != 0 || existNo.Count != 0) return (false, existId, existNo);

            existStuds.Add((student.id, student.name, student.no, null, null));
            existStuds = existStuds.OrderBy(o => string.IsNullOrWhiteSpace(o.no)).ThenBy(o => string.IsNullOrEmpty(o.no) ? 0 : int.Parse(o.no)).ToList();

            //組出新的JSON
            using var memoryStream = new MemoryStream();
            using var writer = new Utf8JsonWriter(memoryStream);
            writer.WriteStartObject();
            //將舊資料寫回
            foreach (var dataElement in classInfo[classId].EnumerateObject())
            {
                if (!dataElement.Name.Equals("students", StringComparison.Ordinal)) dataElement.WriteTo(writer);
            }

            writer.WritePropertyName("students");
            writer.WriteStartArray();
            foreach (var item in existStuds)
            {
                writer.WriteStartObject();
                writer.WriteString("id", item.id);
                if (string.IsNullOrWhiteSpace(item.name)) writer.WriteNull("name");
                else writer.WriteString("name", item.name);
                if (string.IsNullOrWhiteSpace(item.no)) writer.WriteNull("no");
                else writer.WriteString("no", item.no);
                if (string.IsNullOrWhiteSpace(item.groupId)) writer.WriteNull("groupId");
                else writer.WriteString("groupId", item.groupId);
                if (string.IsNullOrWhiteSpace(item.groupName)) writer.WriteNull("groupName");
                else writer.WriteString("groupName", item.groupName);
                writer.WriteEndObject();
            }
            writer.WriteEndArray();
            writer.WriteEndObject();
            writer.Flush();
            try
            {
                var ret = await _azureCosmos
                            .GetCosmosClient()
                            .GetContainer("TEAMModelOS", "School")
                            .ReplaceItemStreamAsync(memoryStream, classId, new PartitionKey($"Class-{schoolId}"));
                return (true, null, null);
            }
            catch (CosmosException ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/joinClass()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            catch (Exception ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/joinClass()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            return (false, null, null);
        }

        /// <summary>
        /// 取得教室資訊，使用classId進行查詢。
        /// </summary>
        /// <returns></returns>
        private async Task<Dictionary<string, JsonElement>> getClassInfoUseId(string schoolId, List<string> classIds)
        {
            try
            {
                if (!(classIds == null || classIds.Count == 0))
                {
                    string queryText = $"SELECT * FROM c WHERE c.code = 'Class-{schoolId}' AND c.id IN ({string.Join(",", classIds.Select(o => $"'{o}'"))})";

                    Dictionary<string, JsonElement> dicClassInfo = new Dictionary<string, JsonElement>();

                    await foreach (Response item in _azureCosmos.GetCosmosClient().GetContainer("TEAMModelOS", "School")
                        .GetItemQueryStreamIterator(queryText: queryText, requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"Class-{schoolId}") }))
                    {
                        using var json = await JsonDocument.ParseAsync(item.ContentStream);
                        if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                        {
                            var classInfos = json.RootElement.GetProperty("Documents").EnumerateArray();
                            while (classInfos.MoveNext())
                            {
                                JsonElement account = classInfos.Current;
                                string id = account.GetProperty("id").GetString();
                                dicClassInfo.Add(id, account.Clone());
                            }
                        }
                    }
                    return dicClassInfo;
                }
                else return null;

            }
            catch (CosmosException ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/getClassInfoUseId()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            catch (Exception ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/getClassInfoUseId()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            return null;
        }

        /// <summary>
        /// 取得教室資訊，使用classNo進行查詢。
        /// </summary>
        /// <returns></returns>
        private async Task<Dictionary<string, JsonElement>> getClassInfoUseNo(string schoolId, List<string> classNos)
        {
            try
            {
                if (!(classNos == null || classNos.Count == 0))
                {
                    string queryText = $"SELECT * FROM c WHERE c.code = 'Class-{schoolId}' AND c.no IN ({string.Join(",", classNos.Select(o => $"'{o}'"))})";

                    Dictionary<string, JsonElement> dicClassInfo = new Dictionary<string, JsonElement>();

                    await foreach (Response item in _azureCosmos.GetCosmosClient().GetContainer("TEAMModelOS", "School")
                        .GetItemQueryStreamIterator(queryText: queryText, requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"Class-{schoolId}") }))
                    {
                        using var json = await JsonDocument.ParseAsync(item.ContentStream);
                        if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                        {
                            var classInfos = json.RootElement.GetProperty("Documents").EnumerateArray();
                            while (classInfos.MoveNext())
                            {
                                JsonElement account = classInfos.Current;
                                string no = account.GetProperty("no").GetString();
                                dicClassInfo.Add(no, account.Clone());
                            }
                        }
                    }
                    return dicClassInfo;
                }
                else return null;

            }
            catch (CosmosException ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/getClassInfoUseId()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            catch (Exception ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/getClassInfoUseId()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            return null;
        }
        
        /// <summary>
        /// 使用學生ID來查詢所屬的教室
        /// </summary>
        /// <param name="schoolId"></param>
        /// <param name="ids"></param>
        /// <returns></returns>
        private async Task<List<JsonElement>> getClassInfoUseStudent(string schoolId, List<string> ids)
        {
            try
            {
                List<string> existId = new List<string>();
                List<JsonElement> listClass = new List<JsonElement>();

                string queryText = $"SELECT DISTINCT VALUE c FROM c JOIN (SELECT VALUE t FROM t IN c.students WHERE t.id IN ({string.Join(",", ids.Select(o => $"'{o}'"))}))";
                List<object> listStudent = new List<object>();

                await foreach (Response item in _azureCosmos.GetCosmosClient().GetContainer("TEAMModelOS", "School")
                    .GetItemQueryStreamIterator(queryText: queryText, requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"Class-{schoolId}") }))
                {
                    using var json = await JsonDocument.ParseAsync(item.ContentStream);
                    if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                    {
                        var Documents = json.RootElement.GetProperty("Documents").EnumerateArray();
                        while (Documents.MoveNext())
                        {
                            listClass.Add(Documents.Current.Clone());
                        }
                    }
                }
                return listClass;
            }
            catch (CosmosException ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/getClassInfoUseStudent()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            catch (Exception ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/getClassInfoUseStudent()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            return null;
        }

        /// <summary>
        /// 批量更新學生資訊，目前支持更新姓名、密碼、座號、性別及教室id。
        /// </summary>
        /// <param name="schoolId"></param>
        /// <param name="students"></param>
        /// <returns></returns>
        private async Task<(List<object> studs, Dictionary<string, List<string>> classDuplNos, List<string> nonexistentIds, List<string> errorIds, Dictionary<string, List<string>> errorNos, List<string> errorClassId)> updateStudents(string schoolId, JsonElement.ArrayEnumerator students)
        {
            try
            {
                //整理輸入的資料->檢查輸入資料有沒有重複座號->取得欲加入的教室資訊->查詢學生並將資料更新並寫入

                //Key:id Value:學生基本資訊
                var studentInfos
                    = new Dictionary<string, (string salt, string pw, string name, string year, string pic, string gender, string mail, string mobile, string classId, string no)>();
                //用於進行座號是否重複查詢時使用
                var classStuds
                    = new Dictionary<string, List<(string id, string salt, string pw, string name, string year, string pic, string gender, string mail, string mobile, string classId, string no)>>();
                //紀錄教室"輸入"的學生座號是否有重複
                var impClassDuplNo = new Dictionary<string, List<string>>();
                //紀錄不存在的學生id
                var nonexistentIds = new List<string>();
                //紀錄跟現有雲端學生座號重複的
                var errorNos = new Dictionary<string, List<string>>();
                //紀錄處理錯誤的id，cosmosdb寫入時錯誤等...
                var errorIds = new List<string>();
                //紀錄沒找到的classId
                var errorClassId = new List<string>();
                //紀錄輸出結果
                var retStuds = new List<object>();

                //整理輸入資料
                while (students.MoveNext())
                {
                    JsonElement student = students.Current;
                    if (student.TryGetProperty("id", out var id))
                    {
                        //確認是否有id欄位，並且確認是否有給pw欄位，若無給或是null empty等，則使用id當密碼。
                        if (!string.IsNullOrWhiteSpace(id.GetString()))
                        {
                            string salt = string.Empty,
                                pw = string.Empty,
                                name = string.Empty,
                                year = string.Empty,
                                gender = string.Empty,
                                mail = string.Empty,
                                mobile = string.Empty,
                                classId = string.Empty,
                                no = string.Empty;

                            //有給pw欄位才進行處理
                            if (student.TryGetProperty("pw", out var tmpPw))
                            {
                                salt = Utils.CreatSaltString(8);
                                pw = !string.IsNullOrWhiteSpace(tmpPw.GetString())
                                    ? Utils.HashedPassword(tmpPw.GetString(), salt)
                                    : Utils.HashedPassword(id.GetString(), salt);
                            }
                            if (student.TryGetProperty("name", out var tmpName))
                            {
                                name = tmpName.GetString();
                            }
                            if (student.TryGetProperty("gender", out var tmpGender))
                            {
                                gender = tmpGender.GetString();
                            }
                            if (student.TryGetProperty("mail", out var tmpMail))
                            {
                                mail = tmpMail.GetString();
                            }
                            if (student.TryGetProperty("mobile", out var tmpMobile))
                            {
                                mobile = tmpMobile.GetString();
                            }
                            if (student.TryGetProperty("year", out var tmpYear))
                            {
                                year = tmpYear.GetString();
                            }
                            if (student.TryGetProperty("classId", out var tmpclassId))
                            {
                                classId = tmpclassId.GetString();
                            }
                            if (student.TryGetProperty("no", out var tmpNo))
                            {
                                no = tmpNo.GetString();
                            }

                            if (!studentInfos.ContainsKey(id.GetString()))
                            {
                                //整理出後面查詢座號要用的參數
                                if (string.IsNullOrWhiteSpace(classId))
                                {
                                    //pic,mail,mobile暫不支持更新
                                    studentInfos.Add(id.GetString(), (salt, pw, name, year, null, gender, null, null, null, null));
                                }
                                else
                                {
                                    if (classStuds.ContainsKey(classId))
                                    {
                                        classStuds[classId].Add((id.GetString(), salt, pw, name, year, null, gender, null, null, classId, no));
                                    }
                                    else
                                    {
                                        classStuds.Add(
                                            classId,
                                            new List<(string id, string salt, string pw, string name, string year, string pic, string gender, string mail, string mobile, string classId, string no)>()
                                            { (id.GetString(), salt, pw, name, year, null, gender, null, null, classId, no) });
                                    }
                                    //pic,mail,mobile暫不支持批量更新
                                    studentInfos.Add(id.GetString(), (salt, pw, name, year, null, gender, null, null, classId, no));
                                }
                                //先將id加進去後面再做刪除動作
                                nonexistentIds.Add(id.GetString());
                            }
                        }
                    }
                }

                //檢查所有輸入的班級資料內，學生座號是否有重複。
                for (int ii = 0; ii <= classStuds.Values.Count - 1; ii++)
                {
                    var duplicateNo = classStuds.ElementAt(ii).Value.GroupBy(o => o.no).Where(o => o.Count() > 1).Select(o => o.Key).ToList();
                    duplicateNo.Remove("");
                    var wrongStuds = classStuds.ElementAt(ii).Value.Where(o => duplicateNo.Contains(o.no)).Select(o => o).ToList();
                    impClassDuplNo.Add(classStuds.ElementAt(ii).Key, wrongStuds.Select(o => o.id).ToList());

                    wrongStuds.ForEach(o => { classStuds.ElementAt(ii).Value.Remove(o); nonexistentIds.Remove(o.id); });
                    classStuds[classStuds.ElementAt(ii).Key] = classStuds.ElementAt(ii).Value;
                }

                //查詢欲加入的教室資訊。
                var classInfo = await getClassInfoUseId(schoolId, classStuds.Keys.ToList());

                //準備查詢db資料
                CosmosContainer cosmosContainer = _azureCosmos.GetCosmosClient().GetContainer("TEAMModelOS", "Student");


                foreach (var item in classStuds)
                {
                    //如果沒有任何學生要更新，則跳過該間教室。
                    if (item.Value.Count == 0) continue;

                    string classId = null, className = null, classNo = null, gradeId = null, periodId = null;

                    //如果教室不存在的話(填錯教室之類的狀況)，則記錄教室的id及學生id。
                    if (classInfo.ContainsKey(item.Key))
                    {
                        classId = item.Key;
                        className = classInfo[item.Key].GetProperty("name").GetString();
                        classNo = classInfo[item.Key].GetProperty("no").GetString();
                        gradeId = classInfo[item.Key].GetProperty("gradeId").GetString();
                        periodId = classInfo[item.Key].GetProperty("periodId").GetString();
                    }
                    else
                    {
                        //沒查到有該間教室的資訊，故將該間教室的ID及學生資料清單記起來，並且跳過不處理該資料。
                        errorClassId.Add(item.Key);
                        item.Value.ForEach(o => nonexistentIds.Remove(o.id));
                        //classStuds.Remove(item.Key);
                        //errorIds.AddRange(item.Value.Select(o => o.id));
                        continue;
                    }

                    //檢查座號是否有重複
                    //若只是改基本資料，該處還是會查到相同的座號。
                    var sutdNos = item.Value.Select(o => o.no).ToList();
                    var existNos = await checkStudNo(schoolId, item.Key, sutdNos);

                    //更新並寫入學生資料
                    if (item.Value.Count != 0)
                    {
                        //查學生的基本資料(該間教室全部的學生)
                        string queryText = $"SELECT * FROM c WHERE c.id IN ({string.Join(",", item.Value.Select(o => $"'{o.id}'"))})";

                        List<JsonElement> listStudent = new List<JsonElement>();

                        await foreach (Response responseItem in cosmosContainer
                            .GetItemQueryStreamIterator(
                                queryText: queryText,
                                requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"Base-{schoolId}") }))
                        {
                            using var json = await JsonDocument.ParseAsync(responseItem.ContentStream);
                            if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                            {
                                var accounts = json.RootElement.GetProperty("Documents").EnumerateArray();
                                while (accounts.MoveNext())
                                {
                                    JsonElement account = accounts.Current;
                                    string id = account.GetProperty("id").GetString();
                                    nonexistentIds.Remove(id);
                                    //舊的座號，基本上不會重複，但可能會是空的
                                    string no = account.GetProperty("no").GetString();

                                    //用來記錄最後更改完的資料
                                    (string salt, string pw, string name, string year, string pic, string gender, string mail, string mobile, string classId, string no) tmpData
                                     = (studentInfos[id].salt, studentInfos[id].pw, studentInfos[id].name, studentInfos[id].year, studentInfos[id].pic, studentInfos[id].gender, studentInfos[id].mail, studentInfos[id].mobile, studentInfos[id].classId, studentInfos[id].no);

                                    bool isUpPwDone = false;
                                    bool isWrong = false;

                                    //開始組Json
                                    using var memoryStream = new MemoryStream();
                                    using var writer = new Utf8JsonWriter(memoryStream);
                                    writer.WriteStartObject();
                                    foreach (var element in account.EnumerateObject())
                                    {
                                        if (isWrong) break;
                                        switch (true)
                                        {
                                            case bool _ when element.Name.Equals("name", StringComparison.Ordinal):
                                                if (string.IsNullOrWhiteSpace(studentInfos[id].name))
                                                {
                                                    element.WriteTo(writer);
                                                    tmpData.name = element.Value.GetString();
                                                }
                                                else
                                                {
                                                    writer.WriteString("name", studentInfos[id].name);
                                                }
                                                break;
                                            case bool _ when element.Name.Equals("pw", StringComparison.Ordinal):
                                            case bool _ when element.Name.Equals("salt", StringComparison.Ordinal):
                                                if (!isUpPwDone && !string.IsNullOrWhiteSpace(studentInfos[id].salt) && !string.IsNullOrWhiteSpace(studentInfos[id].pw))
                                                {
                                                    writer.WriteString("salt", studentInfos[id].salt);
                                                    writer.WriteString("pw", studentInfos[id].pw);
                                                    isUpPwDone = true;
                                                }
                                                break;
                                            case bool _ when element.Name.Equals("gender", StringComparison.Ordinal):
                                                if (string.IsNullOrWhiteSpace(studentInfos[id].gender))
                                                {
                                                    element.WriteTo(writer);
                                                    tmpData.gender = element.Value.GetString();
                                                }
                                                else
                                                {
                                                    writer.WriteString("gender", studentInfos[id].gender);
                                                }
                                                break;
                                            case bool _ when element.Name.Equals("year", StringComparison.Ordinal):
                                                if (string.IsNullOrWhiteSpace(studentInfos[id].year))
                                                {
                                                    element.WriteTo(writer);
                                                    tmpData.year = element.Value.GetString();
                                                }
                                                else
                                                {
                                                    writer.WriteString("year", studentInfos[id].year);
                                                }
                                                break;
                                            case bool _ when element.Name.Equals("classId", StringComparison.Ordinal):
                                                if (string.IsNullOrWhiteSpace(studentInfos[id].classId))
                                                {
                                                    element.WriteTo(writer);
                                                    tmpData.classId = element.Value.GetString();
                                                }
                                                else
                                                {
                                                    writer.WriteString("classId", studentInfos[id].classId);
                                                }
                                                break;
                                            case bool _ when element.Name.Equals("no", StringComparison.Ordinal):
                                                //移除座號的話會給空的
                                                if (studentInfos[id].no.Length == 0)
                                                {
                                                    writer.WriteNull("no");
                                                    tmpData.no = null;
                                                }
                                                else if (string.IsNullOrWhiteSpace(studentInfos[id].no))
                                                {
                                                    element.WriteTo(writer);
                                                    tmpData.no = element.Value.GetString();
                                                }
                                                else
                                                {
                                                    //如果要更新的座號，跟已存在的座號相同，則不進行更新。
                                                    //沒有設定過舊no，或舊座號與新座號不同，則要進行重複座號的檢查。 舊no=null or 舊no!=新no
                                                    if (string.IsNullOrWhiteSpace(no) || (!string.IsNullOrWhiteSpace(no) && !no.Equals(studentInfos[id].no)))
                                                    {
                                                        //如果有檢查到新座號和舊座號重複
                                                        if (existNos.Any(o => o.Item2.Contains(studentInfos[id].no))) //.Contains(studentInfos[id].no))
                                                        {
                                                            if (errorNos.ContainsKey(id))
                                                            {
                                                                errorNos[id].Add(studentInfos[id].no);
                                                            }
                                                            else
                                                            {
                                                                errorNos.Add(id, new List<string>() { studentInfos[id].no });
                                                            }
                                                            isWrong = true;
                                                            break;
                                                        }
                                                    }
                                                    writer.WriteString("no", studentInfos[id].no);
                                                }
                                                break;
                                            case bool _ when element.Name.StartsWith("_", StringComparison.Ordinal):
                                                break;
                                            default:
                                                element.WriteTo(writer);
                                                break;
                                        }
                                    }

                                    //如果有錯誤，如座號重覆等，就會跳過該次更新。
                                    if (isWrong)
                                    {
                                        await writer.DisposeAsync();
                                        continue;
                                    }

                                    //若密碼和鹽沒有更新，就把舊的資料寫回去
                                    if (!isUpPwDone)
                                    {
                                        writer.WriteString("salt", account.GetProperty("salt").GetString());
                                        writer.WriteString("pw", account.GetProperty("pw").GetString());
                                    }
                                    writer.WriteEndObject();
                                    writer.Flush();

                                    try
                                    {
                                        var ret = await cosmosContainer.ReplaceItemStreamAsync(memoryStream, id, new PartitionKey($"Base-{schoolId}"));
                                        //將更新完的id從字典內移除，保留沒查到的。
                                        if (ret.Status == (int)HttpStatusCode.OK)
                                        {
                                            nonexistentIds.Remove(id);
                                            retStuds.Add(new { id, tmpData.name, tmpData.pic, tmpData.year, tmpData.no, classId, classNo, className, gradeId, periodId });
                                        }
                                        else errorIds.Add(id);
                                    }
                                    catch (CosmosException ex)
                                    {
                                        await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/updateStudents()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
                                        errorIds.Add(id);
                                    }
                                    catch (Exception ex)
                                    {
                                        await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/updateStudents()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
                                        errorIds.Add(id);
                                    }
                                }
                            }
                            //將輸入不存在的資料移除。
                            nonexistentIds.ForEach(o => studentInfos.Remove(o));
                        }
                    }
                }
                errorClassId.ForEach(o => impClassDuplNo.Remove(o));

                return (retStuds, impClassDuplNo.Where(o => o.Value.Count != 0).ToDictionary(o => o.Key, o => o.Value), nonexistentIds, errorIds, errorNos, errorClassId);
            }
            catch (CosmosException ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/updateStudents()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            catch (Exception ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/updateStudents()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            return (null, null, null, null, null, null);
        }

        /// <summary>
        /// 批量更新學生所屬教室及學號
        /// </summary>
        /// <param name="schoolId"></param>
        /// <param name="students"></param>
        /// <returns></returns>
        private async Task<Dictionary<string, (string classId, string classNo, string className, string periodId, string gradeId, string name, string no)>> updateClassStudents(string schoolId, Dictionary<string, (string name, string year, string pic, string gender, string mail, string mobile, string classId, string no)> students)
        {
            try
            {
                //最後輸出給前端用的資料
                var retStudentsClassInfo = new Dictionary<string, (string classId, string classNo,string className,string periodId, string gradeId, string name, string no)>();
               
                if (students.Count == 0) return retStudentsClassInfo;

                var tmpStuds = new Dictionary<string, (string name, string year, string pic, string gender, string mail, string mobile, string classId, string no)>(students);

                //透過id查找已加入的教室
                var classInfos = await getClassInfoUseStudent(schoolId, students.Select(o => o.Key).ToList());

                //如果有查到，代表該學生已經加入過某間教室(Class)
                if (classInfos.Count != 0)
                {
                    foreach (var classInfo in classInfos)
                    {
                        //教室資訊的id(UUID)
                        if (classInfo.TryGetProperty("id", out var tmpClassId))
                        {
                            string classId = tmpClassId.GetString();
                            string classNo = null, className = null, periodId = null, gradeId = null;

                            if (classInfo.TryGetProperty("no", out var tmpClassNo)) classNo = tmpClassNo.GetString();
                            if (classInfo.TryGetProperty("name", out var tmpClassName)) className = tmpClassName.GetString();
                            if (classInfo.TryGetProperty("periodId", out var tmpPeriodId)) periodId = tmpPeriodId.GetString();
                            if (classInfo.TryGetProperty("gradeId", out var tmpGradeId)) gradeId = tmpGradeId.GetString();
                            

                            //用以紀錄教室內已存在的學生
                            List<(string id, string name, string no, string groupId, string groupName)> existStudents 
                                = new List<(string id, string name, string no, string groupId, string groupName)>();
                            var Documents = classInfo.GetProperty("students").EnumerateArray();
                            while (Documents.MoveNext())
                            {
                                JsonElement Document = Documents.Current;
                                string studId = null, no = null, name = null, groupId = null, groupName = null;

                                if (Document.TryGetProperty("id", out var tmpId)) studId = tmpId.GetString();
                                if (Document.TryGetProperty("name", out var tmpName)) name = tmpName.GetString();
                                if (Document.TryGetProperty("no", out var tmpNo)) no = tmpNo.GetString();
                                if (Document.TryGetProperty("groupId", out var tmpgroupId)) groupId = tmpgroupId.GetString();
                                if (Document.TryGetProperty("groupName", out var tmpgroupName)) groupName = tmpgroupName.GetString();

                                //檢查輸入資料內學生要加入的教室是否一致
                                if (students.ContainsKey(studId))
                                {
                                    if (students[studId].classId.Equals(classId, StringComparison.Ordinal))
                                    {
                                        //座號及姓名檢查，如果現有資料與欲更新資料不同則進行更新
                                        if (
                                            !(students[studId].no.Equals(no, StringComparison.Ordinal)
                                            && students[studId].name.Equals(name, StringComparison.Ordinal))
                                            )
                                        {
                                            name = students[studId].name;
                                            no = students[studId].no;
                                        }
                                        tmpStuds.Remove(studId);
                                    }
                                    else
                                    {
                                        tmpStuds[studId] = (tmpStuds[studId].name, tmpStuds[studId].year, tmpStuds[studId].pic, tmpStuds[studId].gender, tmpStuds[studId].mail, tmpStuds[studId].mobile, tmpStuds[studId].classId, no);
                                        continue;
                                    }
                                }
                                retStudentsClassInfo.Add(studId, (classId, classNo, className, periodId, gradeId, name, no));
                                existStudents.Add((studId, name, no, groupId, groupName));
                            }
                            existStudents = existStudents.OrderBy(o => string.IsNullOrWhiteSpace(o.no)).ThenBy(o => string.IsNullOrEmpty(o.no) ? 0 : int.Parse(o.no)).ToList();
                            
                            using var memoryStream = new MemoryStream();
                            using var writer = new Utf8JsonWriter(memoryStream);

                            writer.WriteStartObject();
                            //將非students的欄位資料寫回去
                            foreach (var element in classInfo.EnumerateObject())
                            {
                                if (!(element.Name.Equals("students", StringComparison.Ordinal) || element.Name.StartsWith("_")))
                                {
                                    element.WriteTo(writer);
                                }
                            }

                            writer.WritePropertyName("students");
                            writer.WriteStartArray();

                            foreach (var item in existStudents)
                            {
                                writer.WriteStartObject();
                                writer.WriteString("id", item.id);
                                if (string.IsNullOrWhiteSpace(item.name)) writer.WriteNull("name");
                                else writer.WriteString("name", item.name);
                                if (string.IsNullOrWhiteSpace(item.no)) writer.WriteNull("no");
                                else writer.WriteString("no", item.no);
                                if (string.IsNullOrWhiteSpace(item.groupId)) writer.WriteNull("groupId");
                                else writer.WriteString("groupId", item.groupId);
                                if (string.IsNullOrWhiteSpace(item.groupName)) writer.WriteNull("groupName");
                                else writer.WriteString("groupName", item.groupName);
                                writer.WriteEndObject();
                            }

                            writer.WriteEndArray();
                            writer.WriteEndObject();
                            writer.Flush();

                            try
                            {
                                var ret = await _azureCosmos
                                    .GetCosmosClient()
                                    .GetContainer("TEAMModelOS", "School")
                                    .ReplaceItemStreamAsync(memoryStream, classId, new PartitionKey($"Class-{schoolId}"));
                            }
                            catch (CosmosException ex)
                            {
                                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/updateClassStudents()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
                            }
                            catch (Exception ex)
                            {
                                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/updateClassStudents()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
                            }
                        }
                    }
                }
                //始終會加入新教室，除非只是單純的換座號或是姓名
                if (classInfos.Count == 0 || tmpStuds.Count != 0) //透過學生id查找教室，但沒找到任何已加入的教室
                {
                    //使用classId來查詢教室資訊
                    var dicClassInfo = await getClassInfoUseId(schoolId, tmpStuds.Select(o => o.Value.classId).ToList());
                    if (dicClassInfo.Count != 0)
                    {
                        foreach (var classInfo in dicClassInfo)
                        {
                            //用以紀錄教室內已存在的學生
                            List<(string id, string name, string no, string groupId, string groupName)> existStudents
                                = new List<(string id, string name, string no, string groupId, string groupName)>();
                            var Documents = classInfo.Value.GetProperty("students").EnumerateArray();
                            while (Documents.MoveNext())
                            {
                                JsonElement Document = Documents.Current;
                                string studId = null, no = null, name = null, groupId = null, groupName = null;

                                if (Document.TryGetProperty("id", out var tmpId)) studId = tmpId.GetString();
                                if (Document.TryGetProperty("name", out var tmpName)) name = tmpName.GetString();
                                if (Document.TryGetProperty("no", out var tmpNo)) no = tmpNo.GetString();
                                if (Document.TryGetProperty("groupId", out var tmpgroupId)) groupId = tmpgroupId.GetString();
                                if (Document.TryGetProperty("groupName", out var tmpgroupName)) groupName = tmpgroupName.GetString();
                                existStudents.Add((studId, name, no, groupId, groupName));
                            }


                            string classId = classInfo.Key;
                            string classNo = null, className = null, periodId = null, gradeId = null;

                            if (classInfo.Value.TryGetProperty("no", out var tmpClassNo)) classNo = tmpClassNo.GetString();
                            if (classInfo.Value.TryGetProperty("name", out var tmpClassName)) className = tmpClassName.GetString();
                            if (classInfo.Value.TryGetProperty("periodId", out var tmpPeriodId)) periodId = tmpPeriodId.GetString();
                            if (classInfo.Value.TryGetProperty("gradeId", out var tmpGradeId)) gradeId = tmpGradeId.GetString();

                           
                            //如果classId相同，則將該學生加入現有的學生清單內。
                            foreach (var item in tmpStuds)
                            {
                                if (item.Value.classId.Equals(classId, StringComparison.Ordinal))
                                {
                                    existStudents.Add((item.Key, item.Value.name, item.Value.no, null, null));
                                    retStudentsClassInfo.Add(item.Key, (classId, classNo, className, periodId, gradeId, item.Value.name, item.Value.no));
                                }
                            }
                            existStudents = existStudents.OrderBy(o => string.IsNullOrWhiteSpace(o.no)).ThenBy(o => string.IsNullOrEmpty(o.no) ? 0 : int.Parse(o.no)).ToList();


                            #region 組JSON
                            using var memoryStream = new MemoryStream();
                            using var writer = new Utf8JsonWriter(memoryStream);
                            writer.WriteStartObject();
                            //寫入除了students及開頭為"_"的數據資料
                            foreach (var element in classInfo.Value.EnumerateObject())
                            {
                                if (!(element.Name.Equals("students", StringComparison.Ordinal) || element.Name.StartsWith("_")))
                                {
                                    element.WriteTo(writer);
                                }
                            }
                            writer.WritePropertyName("students");
                            writer.WriteStartArray();

                            foreach (var item in existStudents)
                            {
                                writer.WriteStartObject();
                                writer.WriteString("id", item.id);
                                if (string.IsNullOrWhiteSpace(item.name)) writer.WriteNull("name");
                                else writer.WriteString("name", item.name);
                                if (string.IsNullOrWhiteSpace(item.no)) writer.WriteNull("no");
                                else writer.WriteString("no", item.no);
                                if (string.IsNullOrWhiteSpace(item.groupId)) writer.WriteNull("groupId");
                                else writer.WriteString("groupId", item.groupId);
                                if (string.IsNullOrWhiteSpace(item.groupName)) writer.WriteNull("groupName");
                                else writer.WriteString("groupName", item.groupName);
                                writer.WriteEndObject();
                            }

                            writer.WriteEndArray();
                            writer.WriteEndObject();
                            writer.Flush();
                            #endregion

                            try
                            {
                                var ret = await _azureCosmos
                                    .GetCosmosClient()
                                    .GetContainer("TEAMModelOS", "School")
                                    .ReplaceItemStreamAsync(memoryStream, classId, new PartitionKey($"Class-{schoolId}"));
                            }
                            catch (CosmosException ex)
                            {
                                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/updateClassStudents()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
                            }
                            catch (Exception ex)
                            {
                                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/updateClassStudents()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
                            }
                        }
                    }
                    else
                    {
                        //透過教室ID進行查詢，但查不到這間教室
                        await _dingDing.SendBotMsg(
                            $"IES5，" +
                            $"{_option.Location}，" +
                            $"StudentController/updateClassStudents()\n" +
                            $"找不到教室的資訊 {string.Join(",", tmpStuds.Select(o => o.Value.classId))} ",
                            GroupNames.醍摩豆服務運維群組);
                    }
                }

                return retStudentsClassInfo;
            }
            catch (CosmosException ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/updateClassStudents()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            catch (Exception ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/updateClassStudents()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            return null;
        }

        /// <summary>
        /// 創建學生帳號，目前SDK4.0預覽版還不支援批量創建(TransactionalBatch)，待SDK正式發行時在優化此代碼。
        /// </summary>
        /// <param name="userStudents"></param>
        /// <returns>已存在的ID</returns>
        private async Task<(bool isSuc, List<string> existId)> createStudents(List<Student> userStudents)
        {
            var existId = new List<string>();
            var exceptions = new List<Exception>();
            try
            {
                var container = _azureCosmos.GetCosmosClient().GetContainer("TEAMModelOS", "Student");

                Parallel.ForEach(userStudents, async item =>
                {
                    try
                    {
                        await container.CreateItemAsync(item);
                    }
                    catch (CosmosException ex)
                    {
                        if (ex.Status == (int)HttpStatusCode.Conflict) existId.Add(item.id);
                        else exceptions.Add(ex);
                    }
                    catch (Exception ex)
                    {
                        exceptions.Add(ex);
                    }
                });
                if (exceptions.Count == 0) return (true, existId);
                else if (exceptions.Count > 1) throw new AggregateException(exceptions);
                else if (exceptions.Count == 1) throw exceptions.Single();
            }
            catch (AggregateException ex)
            {
                Console.WriteLine(ex.Message);
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/createStudents()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/createStudents()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            return (false, existId);
        }

        /// <summary>
        /// 批量重設密碼，若無給密碼則預設使用學號當密碼
        /// </summary>
        /// <param name="schoolId"></param>
        /// <param name="students"></param>
        /// <returns>不存在的id</returns>
        private async Task<List<string>> resetPW(string schoolId, JsonElement.ArrayEnumerator students)
        {
            try
            {
                //取得前端欲重置的id與pw
                Dictionary<string, (string salt, string pw)> studentsInfo = new Dictionary<string, (string salt, string pw)>();

                while (students.MoveNext())
                {
                    JsonElement student = students.Current;
                    if (student.TryGetProperty("id", out var id))
                    {
                        //確認是否有id欄位，並且確認是否有給pw欄位，若無給或是null empty等，則使用id當密碼
                        if (!string.IsNullOrWhiteSpace(id.GetString()))
                        {
                            string salt = Utils.CreatSaltString(8);
                            string pw = student.TryGetProperty("pw", out var tmpPw) && !string.IsNullOrWhiteSpace(tmpPw.GetString())
                                ? Utils.HashedPassword(tmpPw.GetString(), salt)
                                : Utils.HashedPassword(id.GetString(), salt);
                            if (!studentsInfo.ContainsKey(id.GetString())) studentsInfo.Add(id.GetString(), (salt, pw));
                        }
                    }
                }

                CosmosContainer cosmosContainer = _azureCosmos.GetCosmosClient().GetContainer("TEAMModelOS", "Student");
                //查學生的基本資料
                string queryText = $"SELECT * FROM c WHERE c.code = 'Base-{schoolId}' AND c.id IN ({string.Join(",", studentsInfo.Select(o => $"'{o.Key}'"))})";

                List<JsonElement> listStudent = new List<JsonElement>();

                await foreach (Response item in cosmosContainer
                    .GetItemQueryStreamIterator(
                        queryText: queryText,
                        requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"Base-{schoolId}") }))
                {
                    using var json = await JsonDocument.ParseAsync(item.ContentStream);
                    if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                    {
                        var accounts = json.RootElement.GetProperty("Documents").EnumerateArray();
                        while (accounts.MoveNext())
                        {
                            JsonElement account = accounts.Current;
                            string id = account.GetProperty("id").GetString();
                            bool isGetNameSuc = account.TryGetProperty("name", out var name);
                            bool isGetClassIdSuc = account.TryGetProperty("classId", out var classId);
                            bool isGetNoSuc = account.TryGetProperty("no", out var no);

                            using var memoryStream = new MemoryStream();
                            using var writer = new Utf8JsonWriter(memoryStream);
                            writer.WriteStartObject();
                            foreach (var element in account.EnumerateObject())
                            {
                                if (
                                     !(
                                        element.Name.Equals("salt", StringComparison.Ordinal)
                                     || element.Name.Equals("pw", StringComparison.Ordinal)
                                     || element.Name.StartsWith("_", StringComparison.Ordinal)))
                                {
                                    element.WriteTo(writer);
                                }
                            }
                            writer.WriteString("salt", studentsInfo[id].salt);
                            writer.WriteString("pw", studentsInfo[id].pw);
                            writer.WriteEndObject();
                            writer.Flush();

                            try
                            {
                                var ret = await cosmosContainer.ReplaceItemStreamAsync(memoryStream, id, new PartitionKey($"Base-{schoolId}"));
                            }
                            catch (CosmosException ex)
                            {
                                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/resetPW()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
                            }
                            catch (Exception ex)
                            {
                                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/resetPW()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
                            }

                            //將更新完的id從字典內移除
                            studentsInfo.Remove(id);
                        }
                    }
                }
                return studentsInfo.Select(o => o.Key).ToList();
            }
            catch (Exception ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/resetPW()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
            }
            return null;
        }

        /// <summary>
        /// 取得該教室的學生座號，若有給座號LIST，則座號存在才會被查到；反之，若沒給則會將該間教室所有座號抓出來。
        /// </summary>
        /// <param name="schoolId"></param>
        /// <param name="classId"></param>
        /// <param name="nos"></param>
        /// <returns></returns>
        private async Task<List<(string id,string no)>> checkStudNo(string schoolId, string classId, List<string> nos = null)
        {
            List<(string id, string no)> ret = new List<(string id, string no)>();

            string queryText = $"SELECT c.id, c.no FROM c WHERE c.classId = '{classId}' AND c.code = 'Base-{schoolId}'";
            if (nos != null) queryText += $"AND c.no IN ({string.Join(",", nos.Select(o => $"'{o}'"))})";
            await foreach (Response item in _azureCosmos.GetCosmosClient().GetContainer("TEAMModelOS", "Student")
                .GetItemQueryStreamIterator(queryText: queryText, requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"Base-{schoolId}") }))
            {
                using var json = await JsonDocument.ParseAsync(item.ContentStream);
                if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                {
                    var classInfos = json.RootElement.GetProperty("Documents").EnumerateArray();
                    while (classInfos.MoveNext())
                    {
                        JsonElement account = classInfos.Current;
                        string id = account.GetProperty("id").GetString();
                        string no = account.GetProperty("no").GetString();
                        ret.Add((id, no));
                    }
                }
            }
            return ret;
        }

        private struct studCreateInfo
        {
            public studCreateInfo(string id, string name, string gender, string year, string pw, string classId, string no)
            {
                this.id = id;
                this.name = name;
                this.gender = gender;
                this.year = year;
                this.pw = pw;
                this.classId = classId;
                this.no = no;
            }
            public string id { get; }
            public string name { get; }
            public string gender { get; }
            public string year { get; }
            public string pw { get; }
            public string classId { get; }
            public string no { get; }
        }

        /// <summary>
        /// 學生登入
        /// </summary>
        /// <param name = "request" ></ param >
        [AllowAnonymous]
        [HttpPost("login")]
        public async Task<IActionResult> Login(JsonElement request)
        {
            try
            {
                var client = _azureCosmos.GetCosmosClient();
                //參數取得
                if (!request.TryGetProperty("school_code", out JsonElement school_code)) return BadRequest();
                if (!request.TryGetProperty("id", out JsonElement id)) return BadRequest();
                if (!request.TryGetProperty("pw", out JsonElement pw)) return BadRequest();  

                var response = await client.GetContainer("TEAMModelOS", "Student").ReadItemStreamAsync(id.GetString(), new PartitionKey($"Base-{school_code.GetString().ToLower()}"));
                if (response.Status == 200)
                {
                    var rjson = await JsonDocument.ParseAsync(response.ContentStream);
                    rjson.RootElement.TryGetProperty("salt", out JsonElement salt);
                    rjson.RootElement.TryGetProperty("pw", out JsonElement dbpw);
                    rjson.RootElement.TryGetProperty("name", out JsonElement name);
                    rjson.RootElement.TryGetProperty("picture", out JsonElement picture);                    

                    var HashedPW = Utils.HashedPassword(pw.ToString(), salt.ToString());
                    if (HashedPW.Equals(dbpw.GetString()))
                    {
                        string classid = string.Empty;
                        object classinfo = null;
                        List<object> courses = new List<object>();
                        //去學校找出所屬校本預設班級信息
                        var query = $"SELECT c.code, c.id, c.name, c.periodId, c.gradeId FROM c JOIN cs IN c.students WHERE cs.id = '{id}'";
                        var school = client.GetContainer("TEAMModelOS", "School");
                        await foreach (var item in school.GetItemQueryStreamIterator(queryText: query, requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"Class-{school_code}") }))
                        {
                            using var json = await JsonDocument.ParseAsync(item.ContentStream);
                            if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                            {
                                foreach (var obj in json.RootElement.GetProperty("Documents").EnumerateArray())
                                {
                                    classid = obj.GetProperty("id").GetString();
                                    classinfo = obj.ToObject<object>();                                    
                                }
                            }
                        }
                        //找出所屬班級的課程列表                                    
                        var queryc = $"SELECT VALUE cc.course FROM c JOIN cc IN c.courses WHERE c.id = '{classid}'";
                        await foreach (var item in school.GetItemQueryStreamIterator(queryText: queryc, requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"CourseManagement-{school_code}") }))
                        {
                            using var json = await JsonDocument.ParseAsync(item.ContentStream);
                            if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                            {
                                foreach (var obj in json.RootElement.GetProperty("Documents").EnumerateArray())
                                {
                                    courses.Add(obj.ToObject<object>());
                                }
                            }
                        }
                        // BLOB(學校，唯讀)
                        var (blob_uri, blob_sas) = _azureStorage.GetBlobContainerSAS(school_code.GetString().ToLower(), BlobContainerSasPermissions.Read);
                        //換取AuthToken，提供給前端
                        var auth_token = JwtAuthExtension.CreateAuthToken(_option.HostName, id.GetString(), name.GetString(), picture.GetString(), _option.JwtSecretKey, schoolID: school_code.GetString(), roles: new[] { "student" });
                        return Ok(new { error = 0, auth_token, blob_uri, blob_sas, classinfo, courses });
                    }
                    else
                    {
                        return Ok(new { error = 1, message = "账号或密码错误" });
                    }
                }
                else
                {
                    return Ok(new { error = 2, message = "無此帳號存在" });
                }                
            }
            catch (Exception ex)
            {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，StudentController/login()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
                return BadRequest();
            }
        }
    }
}
